% This function computes the negative loglikelihood function for the population size estimation
% model as stated in Equation (13) of [1] for a given set of parameters and measurements.
% For this function we assume that the actual arrival/departure times of
% the discovered individuals are known; the likelihood function
% computed is given in Section I-B of [2].
%
% Input:
% parameters is a two-element vector where the first element (resp. second) is the
% estimate of the alpha (resp. beta) parameter of the gamma distribution
% modeling the contact rates.
% contMat is an SxM matrix where element ij is equal to the number of times
% that agent j contacts individual i, for 1 <= i <= S and 1 <= j <= M.
% agentEntDepTimes is an Mx2 matrix where the first column is the arrival
% times of the agents to the event and the second column is the departure
% times of the agents from the event.
% allEntDepTimes is a two-column matrix containing samples of the
% arrival/departure times of the individuals to/from the event. Note that
% these samples could represent the actual distribution or an approximate
% distribution of the arrival/departure times.
% discIndEntDepTimes is an Sx2 matrix containing the actual arrival and departure
% times of all the S discovered individuals to/from the event.
%
% Output:
% NhatValue is the estimated population size for the given set of parameters and
% measurements
% logLike is the estimated negative log likelihood value given the set of
% parameters and measurements
%
% [1]: "Opportunistic Sampling for Joint Population Size and Density Estimation",
% F. M. Naini, O. Dousse, P. Thiran, and M. Vetterli, IEEE Transactions on Mobile
% Computing, Volume 14, Issue 12, 2015, available at: http://infoscience.epfl.ch/ and http://rr.epfl.ch/
% [2]: "Opportunistic Sampling for Joint Population Size and Density
% Estimation: Supplementary Notes", F. M. Naini, O. Dousse, P. Thiran,
% and M. Vetterli, 2015, available at: http://infoscience.epfl.ch/ and http://rr.epfl.ch/

function [logLike, NhatValue] = PopEstLogLikeKnownTimes(parameters, contMat, discIndEntDepTimes, agentEntDepTimes, allEntDepTimes)

[S, M] = size(contMat);

% the alpha and beta parameters
alphaValue = parameters(1);
betaValue = parameters(2);

% estimating commonStayTimeVisible, which is equal to time during which both
% a discovered individual and an agent are in the event
commonStayTimeVisible = zeros(S, M);
for indIter = 1:S
    for agentIter = 1:M
        commonTime = max(min(discIndEntDepTimes(indIter, 2), agentEntDepTimes(agentIter, 2)) - max(discIndEntDepTimes(indIter, 1), agentEntDepTimes(agentIter, 1)), 0);
        if commonTime > 0
            commonStayTimeVisible(indIter, agentIter) = commonTime;
        end
    end
end

% estimating commonStayTime, which is equal to time during which both
% an individual and an agent are in the event. For the individuals we use
% here the arrival/departure times in allEntDepTimes, which are either the actual
% times or drawn from the actual or approximate distribution
commonStayTime = zeros(size(allEntDepTimes, 1), M);
for indIter = 1:size(allEntDepTimes, 1)
    for agentIter = 1:M
        commonTime = max(min(allEntDepTimes(indIter, 2), agentEntDepTimes(agentIter, 2)) - max(allEntDepTimes(indIter, 1), agentEntDepTimes(agentIter, 1)), 0);
        if commonTime > 0
            commonStayTime(indIter, agentIter) = commonTime;
        end
    end
end

samplesOverlapTime = sum(commonStayTime, 2);
sumOverlapTime = sum(commonStayTimeVisible, 2);
sumDet = sum(contMat, 2);

% computing the probability of discovering an individual by any agent for
% the given set of parameters
probDetValue = 1 - mean((betaValue ./ (betaValue + samplesOverlapTime)) .^ alphaValue);

% the ML estimate for population size given the current
% values of alpha and beta
NhatValue = ceil(S / probDetValue);

%% estimating the loglikelihood of the undiscovered individuals
% part 1 represents the loglikelihood of nchoosek (N, S) in the likelihood equation in Section I-B of [2]
logLikeUnObs_part1 = gammaln(NhatValue + 1) - gammaln(NhatValue - S + 1) - gammaln(S + 1);
% part 2 represents the loglikelihood of the expection including the (N-S) exponent
% in the likelihood equation in Section I-B of [2]
logLikeUnObs_part2 = (NhatValue - S) * log(1 - probDetValue);
if isnan(logLikeUnObs_part2)
    % in this case probDetValue=1 and NhatValue=S
    logLikeUnObs_part2 = 0;
end
logLikeUnObs = logLikeUnObs_part1 + logLikeUnObs_part2;

%% estimating the loglikelihood of the discovered individuals
% part 1 represents the loglikelihood of (beta^alpha/gamma(alpha)) for all
% the S discovered individuals in the likelihood equation in Section I-B of [2] 
logLikeObs_part1 = S * alphaValue * log(betaValue) - S * gammaln(alphaValue);
% part 2 represents the loglikelihood of (gamma(alpha+k_i)) for all
% the S discovered individuals in the likelihood equation in Section I-B of [2]
logLikeObs_part2 = sum(gammaln(alphaValue + sumDet));
% part 3 represents the loglikelihood of ((beta+sum_{j=1}^{M}t_{at,dt}^j)^(alpha+k_i))
% for all the S discovered individuals in the likelihood equation in Section I-B of [2]
logLikeObs_part3 = - (alphaValue + sumDet)'*log(betaValue+sumOverlapTime);
% part 4 represents the loglikelihood of (prod_{j=1}^{M}t_{at,dt}^j)^(k_ij))
% for all the S discovered individuals in the likelihood equation in Section I-B of [2]
logLikeObs_part4 = contMat .* log(commonStayTimeVisible);
logLikeObs_part4(isnan(logLikeObs_part4)) = 0;
logLikeObs_part4 = sum(sum(logLikeObs_part4));
% part 5 represents the loglikelihood of (prod_{j=1}^{M}1/(k_ij!))
% for all the S discovered individuals in the likelihood equation in Section I-B of [2]
logLikeObs_part5 = - sum(sum(gammaln(contMat + 1)));

logLikeObs = logLikeObs_part1 + logLikeObs_part2 + logLikeObs_part3 + logLikeObs_part4 + logLikeObs_part5;

% computing the total loglikelihood
logLike = logLikeObs + logLikeUnObs;

% negating
logLike = - logLike;