% This function simulates contact traces between agents and individuals
% according to the population size estimation model in [1].
%
% Input:
% N is population size (i.e., number of individuals)
% M is the number of agents
% alpha0 and beta0 are parameters of the gamma distribution modeling the contact
% rates between the individuals and agents
% eventDuration is the duration of the experiment (e.g., event duration)
% or observation window
%
% Output:
% contTrace is a cell containing M matrices where each matrix corresponds to the
% traces of one agent. Each matrix has two columns where the first column
% is the ID of the individual contacted by the agent and the second column
% is the time of contact. The IDs of individuals are numbered from 1 to S
% where S <= N is the total number of discovered individuals. The time of contacts are
% between 0 and eventDuration. The rows of the matrix are sorted according
% to the time of contact.
% agentEntDepTimes is an Mx2 matrix where the first column is the arrival
% times of the agents to the event and the second column is the departure
% times of the agents from the event.
% allEntDepTimes is an Nx2 containing the arrival and departure times of all
% the N individuals to/from the event.
% discIndEntDepTimes is an Sx2 matrix containing the arrival and departure
% times of all the S discovered individuals to/from the event.
%
% [1]: "Opportunistic Sampling for Joint Population Size and Density Estimation",
% F. M. Naini, O. Dousse, P. Thiran, and M. Vetterli, IEEE Transactions on Mobile
% Computing, Volume 14, Issue 12, 2015, available at: http://infoscience.epfl.ch/ and http://rr.epfl.ch/

function [contTrace, S, agentEntDepTimes, allEntDepTimes, discIndEntDepTimes] = traceSimulate1(alpha0, beta0, eventDuration, M, N)

%% simulating agents' arrival and departure times
% feel free to modify this part
agentEntDepTimes = zeros(M, 2);
agentEntDepTimes(:, 1) = rand(M, 1) * eventDuration / 4;
agentEntDepTimes(:, 2) = eventDuration * 3 / 4 + rand(M, 1) * eventDuration / 4;

%% simulating individuals' arrival and departure times
% feel free to modify this part
allEntDepTimes = zeros(N, 2);
%allEntDepTimes(:,1)=exprnd(eventDuration/5,N,1);
allEntDepTimes(:, 1) = gamrnd(3, eventDuration / 15, [N, 1]);
allEntDepTimes(allEntDepTimes(:, 1) > eventDuration * 0.95, 1) = eventDuration * 0.95;
allEntDepTimes(:, 2) = allEntDepTimes(:, 1) + randn(N, 1) * eventDuration / 5 + 1 / 2 * eventDuration;
allEntDepTimes(allEntDepTimes(:, 2) > eventDuration, 2) = eventDuration;
allEntDepTimes = sort(allEntDepTimes, 2);
allEntDepTimes(allEntDepTimes(:, 1) < 0, 1) = 1;

%% simulating the contact rates
contRates = gamrnd(alpha0, 1 / beta0, [N, 1]);

%% simulating the contacts
isDisovered = zeros(N, 1); %to identify the individuals that have been discovered by the agents

contTrace = cell(M, 1); 
for agentIter = 1:M
    % iteration on every agent. Building the trace for this agent.
    agentTrace = [];
    for indIter = 1:N
        % iteration on every individual. Building the trace for this individual and the agent.
        % first computing the duration of time that both the individual and the
        % agent are at the event
        commonOverlapTime = max(min(allEntDepTimes(indIter, 2), agentEntDepTimes(agentIter, 2)) - max(allEntDepTimes(indIter, 1), agentEntDepTimes(agentIter, 1)), 0);
        if commonOverlapTime > 0
            % if commonOverlapTime>0 then there is possibility of contact  between the two
            contNum = poissrnd(commonOverlapTime * contRates(indIter)); % simulating the number of contacts
            if contNum > 0
                % if we have contacts, then we simulate the time of contacts between the individual and the agent
                contTimes = rand(contNum, 1) * commonOverlapTime + max(allEntDepTimes(indIter, 1), agentEntDepTimes(agentIter, 1));
                isDisovered(indIter) = 1; % indicating that this individual is discovered
                agentTrace = [agentTrace; [ones(length(contTimes), 1) * indIter contTimes]];
            end
        end
    end
 
    % sorting the agent's trace based on time
    [~, index] = sort(agentTrace(:, 2));
    agentTrace = agentTrace(index, :);
    contTrace{agentIter} = agentTrace; % saving the agent's trace
 
end

S = sum(isDisovered == 1); % calculating S
discIndEntDepTimes = allEntDepTimes(isDisovered == 1, :); % calculating the arrrival/departure times of the discovered individuals

newIndice = cumsum(isDisovered);
% renaming the IDs of the discovered individuals to 1--S.
for agentIter = 1:M
    agentTrace = contTrace{agentIter, 1};
    agentTrace(:, 1) = newIndice(agentTrace(:, 1));
    contTrace{agentIter, 1} = agentTrace;
end